@@ -1,4 +1,5 @@
 import logging
+import re
 import uuid
 
 import orjson
@@ -33,6 +34,22 @@ def _transform(ops, filename):
             yield result
 
 
+def infer_type(key, value):
+    if key == 'id':
+        rtype = 'TEXT UNIQUE'
+    elif key == 'number_observed':
+        rtype = 'NUMERIC'
+    elif isinstance(value, int):
+        rtype = 'NUMERIC'
+    elif isinstance(value, float):
+        rtype = 'REAL'
+    elif isinstance(value, list):
+        rtype = 'TEXT'
+    else:
+        rtype = 'TEXT'
+    return rtype
+
+
 class SqlStorage:
     def __init__(self):
         self.connection = None  # Python DB API connection object
@@ -153,6 +170,11 @@ class SqlStorage:
         self.connection.commit()
         cursor.close()
 
+    def _add_column(self, tablename, prop_name, prop_type):
+        stmt = f'ALTER TABLE "{tablename}" ADD COLUMN "{prop_name}" {prop_type};'
+        logger.debug('new_property: "%s"', stmt)
+        self._execute(stmt)
+
     def _create_view(self, viewname, select, sco_type, deps=None, cursor=None):
         # This is DB-specific
         raise NotImplementedError('Storage._create_view')
@@ -565,9 +587,24 @@ class SqlStorage:
         validate_name(oldname)
         validate_name(newname)
         view_type = self.table_type(oldname)
+        view_def = self._get_view_def(oldname)
         cursor = self._execute('BEGIN;')
-        self._execute(f'ALTER VIEW "{oldname}" RENAME TO "{newname}";', cursor)
+
+        # Need to remove `newname` if it already exists
+        self._execute(f'DROP VIEW IF EXISTS "{newname}";', cursor)
+        self._drop_name(cursor, newname)
+
+        # Update membership table
+        self._execute(f"UPDATE __membership SET var = '{newname}' WHERE var = '{oldname}';", cursor)
+
+        # Now do the rename
+        qry = re.sub(f'var = \'{oldname}\'',  # This is an ugly hack
+                     f'var = \'{newname}\'',
+                     view_def)
+        self._create_view(newname, qry, view_type, cursor=cursor)
+        self._execute(f'DROP VIEW IF EXISTS "{oldname}"', cursor)
         self._drop_name(cursor, oldname)
         self._new_name(cursor, newname, view_type)
+
         self.connection.commit()
         cursor.close()
